--- ksysguard/ksysguardd/FreeBSD/CPU.c.orig	2005-01-14 13:12:41.000000000 +0100
+++ ksysguard/ksysguardd/FreeBSD/CPU.c	2005-01-14 13:48:35.000000000 +0100
@@ -22,7 +22,9 @@
 #include <osreldate.h>
 
 #include <sys/types.h>
-#if __FreeBSD_version < 500101
+#if __DragonFly__
+	#include <kinfo.h>
+#elif __FreeBSD_version < 500101
 	#include <sys/dkstat.h>
 #else
 	#include <sys/resource.h>
@@ -44,6 +46,15 @@
 
 unsigned long cp_time_offset;
 
+#if __DragonFly__
+#define CPUSTATES 5
+#define CP_USER 0
+#define CP_NICE 1
+#define CP_SYS  2
+#define CP_INTR 3
+#define CP_IDLE 4
+struct kinfo_cputime cpu_time;
+#endif
 long cp_time[CPUSTATES];
 long cp_old[CPUSTATES];
 long cp_diff[CPUSTATES];
@@ -61,6 +72,10 @@
 			printCPUSysInfo, sm);
 	registerMonitor("cpu/idle", "integer", printCPUIdle,
 			printCPUIdleInfo, sm);
+#if __DragonFly__
+	registerMonitor("cpu/intr", "integer", printCPUIntr,
+			printCPUIntrInfo, sm);
+#endif
 
 	updateCpuInfo();
 }
@@ -74,7 +89,18 @@
 updateCpuInfo(void)
 {
 	size_t len = sizeof(cp_time);
+#if __DragonFly__
+	if (kinfo_get_sched_cputime(&cpu_time)) {
+		return;
+	}
+	cp_time[CP_USER] = cpu_time.cp_user;
+	cp_time[CP_NICE] = cpu_time.cp_nice;
+	cp_time[CP_SYS] = cpu_time.cp_sys;
+	cp_time[CP_INTR] = cpu_time.cp_intr;
+	cp_time[CP_IDLE] = cpu_time.cp_idle;
+#else
         sysctlbyname("kern.cp_time", &cp_time, &len, NULL, 0);
+#endif
         percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
 	return (0);
 }
@@ -127,6 +153,21 @@
 	fprintf(CurrentClient, "CPU Idle Load\t0\t100\t%%\n");
 }
 
+#if __DragonFly__
+
+void
+printCPUIntr(const char* cmd)
+{
+	fprintf(CurrentClient, "%d\n", cpu_states[CP_INTR]/10);
+}
+
+void
+printCPUIntrInfo(const char* cmd)
+{
+	fprintf(CurrentClient, "CPU Interrupt Load\t0\t100\t%%\n");
+}
+
+#endif
 
 /* The part ripped from top... */
 /*
